from AccessControl import getSecurityManager
from Acquisition import aq_inner
from Products.Five.browser import BrowserView
from collective.cover.browser.cover import Standard
from eea.climateadapt.vocabulary import BIOREGIONS
from eea.climateadapt.vocabulary import SUBNATIONAL_REGIONS
from eea.climateadapt.vocabulary import ace_countries_dict
from plone import api
from plone.app.iterate.browser.control import Control
from plone.app.iterate.interfaces import ICheckinCheckoutPolicy
from plone.app.iterate.interfaces import IIterateAware
from plone.app.iterate.interfaces import IObjectArchiver
from plone.app.iterate.interfaces import IWorkingCopy
from plone.app.iterate.permissions import CheckinPermission
from plone.app.iterate.permissions import CheckoutPermission
from zExceptions import NotFound
import json
import logging

logger = logging.getLogger('eea.climateadapt')


class AceViewApi(object):
    def html2text(self, html):
        if not isinstance(html, basestring):
            return u""
        portal_transforms = api.portal.get_tool(name='portal_transforms')
        data = portal_transforms.convertTo('text/plain',
                                           html, mimetype='text/html')
        text = data.getData()
        return text.strip()

    def linkify(self, text):
        if not text:
            return

        if text.startswith('/') or text.startswith('http'):
            return text

        return "http://" + text

    def get_websites(self):
        """ This returns a list of websites. Because of BBB, we need to treat
        them in various ways
        """
        websites = self.context.websites
        storage_type = getattr(self.context, 'storage_type', None)

        result = []
        for link in websites:
            if (storage_type == 'MAPLAYER'):
                url = '/tools/map-viewer?layerid=' + link
                if (self.context.search_type == 'MAPGRAPHDATASET'):
                    result.append({'url': url,
                                   'title': 'View map ' + self.context.title})
                else:
                    result.append({'url': self.linkify(link), 'title': link})
            else:
                result.append({'url': self.linkify(link), 'title': link})

        return result

    def get_files(self):
        return [r.to_object for r in self.context.relatedItems] \
            + self.context.contentValues({'portal_type': 'File'})

    def _render_geochar_element(self, value):
        value = BIOREGIONS[value]
        return u"<p>{0}</p>".format(value)
        # if value == 'Global':
        #     return value + u"<br/>"
        # else:
        #     return value + u":<br/>"

    def _render_geochar_macrotrans(self, value):
        tpl = u"<div class='sidebar_bold'>Macro-Transnational region: <br/><p>{0}</p></div>"
        return tpl.format(u", ".join([BIOREGIONS[x] for x in value]))

    def _render_geochar_biotrans(self, value):
        tpl = u"<div class='sidebar_bold'>Biographical regions: <br/><p>{0}</p></div>"
        return tpl.format(u", ".join([BIOREGIONS.get(x, x) for x in value]))

    def _render_geochar_countries(self, value):
        tpl = u"<div class='sidebar_bold'>Countries:<br/><p>{0}</p></div>"
        return tpl.format(u", ".join(self.get_countries(value)))

    def _render_geochar_subnational(self, value):
        tpl = u"<div class='sidebar_bold'>Sub Nationals: <br/><p>{0}</p></div>"
        # a list like: ['SUBN_Marche__IT_']

        out = []
        for line in value:
            line = line.encode('utf-8')
            if line in SUBNATIONAL_REGIONS:
                out.append(SUBNATIONAL_REGIONS[line])
                continue
            else:
                logger.error("Subnational region not found: %s", line)

        text = u", ".join([x.decode('utf-8') for x in out])
        return tpl.format(text)

    def _render_geochar_city(self, value):
        text = value
        if isinstance(value, (list, tuple)):
            text = u", ".join(value)
        return u"<div class='sidebar_bold'>City: <br/><p>{0}</p></div>".format(text)

    def render_geochar(self, value):
        # value is a mapping such as:
        # u'{"geoElements":{"element":"EUROPE",
        #                   "macrotrans":["TRANS_MACRO_ALP_SPACE"],
        #                   "biotrans":[],
        #                   "countries":[],
        #                   "subnational":[],
        #                   "city":""}}'

        if not value:
            return u""

        value = json.loads(value)

        out = []
        order = ['element', 'macrotrans', 'biotrans',
                 'countries', 'subnational', 'city']

        for key in order:
            element = value['geoElements'].get(key)
            if element:
                renderer = getattr(self, "_render_geochar_" + key)
                out.append(renderer(element))

        return u" ".join(out)

    def link_to_original(self):
        """ Returns link to original object, to allow easy comparison
        """
        site = "http://climate-adapt-old.eea.europa.eu"
        if hasattr(self.context, '_aceitem_id'):
            return ("{0}/viewaceitem?aceitem_id={1}".format(
                site, self.context._aceitem_id))
        if hasattr(self.context, '_acemeasure_id'):
            return ("{0}/viewmeasure?ace_measure_id={1}".format(
                site, self.context._acemeasure_id))
        if hasattr(self.context, '_aceproject_id'):
            return (
                "{0}/projects1?ace_project_id={1}".format(
                    site, self.context._aceproject_id))

    def get_countries(self, country_list):
        return [ace_countries_dict.get(x, x) for x in country_list]

    def type_label(self):
        from eea.climateadapt.vocabulary import _datatypes
        d = dict(_datatypes)
        return d[self.context.search_type]

    def governance_level(self):
        if self.context.governance_level is None:
            return ""
        if len(self.context.governance_level) == 0:
            return ""

        from eea.climateadapt.vocabulary import _governance
        d = dict(_governance)
        return [d.get(b) for b in self.context.governance_level]


class ViewAceItem(BrowserView):
    """ Redirection view for /viewaceitem?aceitem_id=..."
    """

    def __call__(self, REQUEST):

        aceitem_id = REQUEST.get('aceitem_id')
        if aceitem_id:
            try:
                aceitem_id = int(aceitem_id)
            except ValueError:
                raise NotFound
        else:
            raise NotFound

        return redirect(self.context, REQUEST, 'aceitem_id', aceitem_id)


class ViewAceMeasure(BrowserView):
    """ Redirection view for /viewacemeasure?ace_measure_id=..."
    """

    def __call__(self, REQUEST):

        acemeasure_id = REQUEST.get('ace_measure_id')
        if (not acemeasure_id) or (not acemeasure_id.isdigit()):
            raise NotFound

        acemeasure_id = int(acemeasure_id)

        return redirect(self.context, REQUEST, 'acemeasure_id', acemeasure_id)


class ViewAceProject(BrowserView):
    """ Redirection view for /project1?ace_project_id=..."
    """

    def __call__(self, REQUEST):

        aceproject_id = REQUEST.get('ace_project_id')
        if not aceproject_id or not aceproject_id.isdigit():
            raise NotFound

        aceproject_id = int(aceproject_id)

        return redirect(self.context, REQUEST, 'aceproject_id', aceproject_id)


def redirect(site, REQUEST, key, itemid):
    portal_catalog = site.portal_catalog
    brains = portal_catalog({key: itemid})
    if not brains:
        raise NotFound
    ob = brains[0].getObject()
    return REQUEST.RESPONSE.redirect(ob.absolute_url(), status=301)


class CoverNoTitleView(Standard):
    """ A template for covers that doesn't include the title
    """

    def __call__(self):
        return self.index()


class IterateControl(Control):
    """ Better behaviour for plone.app.iterate
    """

    def is_checkout(self):
        """ Is this object a checkout? Used by CCA for workflow guards
        """
        context = aq_inner(self.context)

        if not IIterateAware.providedBy(context):
            return False

        archiver = IObjectArchiver(context)
        if not archiver.isVersionable():
            return False

        if IWorkingCopy.providedBy(context):
            return True

        return False

    def checkin_allowed(self):
        """ Overrided to check for the checkin permission, as it should be normal
        """

        context = aq_inner(self.context)
        checkPermission = getSecurityManager().checkPermission

        if not IIterateAware.providedBy(context):
            return False

        archiver = IObjectArchiver(context)
        if not archiver.isVersionable():
            return False

        if not IWorkingCopy.providedBy(context):
            return False

        policy = ICheckinCheckoutPolicy(context, None)
        if policy is None:
            return False

        original = policy.getBaseline()
        if original is None:
            return False

        checkPermission = getSecurityManager().checkPermission
        if not checkPermission(CheckinPermission, original):
            return False

        return True

    def checkout_allowed(self):
        """ Overrided to check for the checkout permission, as it is normal
        """
        context = aq_inner(self.context)

        if not IIterateAware.providedBy(context):
            return False

        archiver = IObjectArchiver(context)
        if not archiver.isVersionable():
            return False

        policy = ICheckinCheckoutPolicy(context, None)
        if policy is None:
            return False

        if policy.getWorkingCopy() is not None:
            return False

        # check if its is a checkout
        if policy.getBaseline() is not None:
            return False

        checkPermission = getSecurityManager().checkPermission
        if not checkPermission(CheckoutPermission, context):
            return False

        return True
